#ifndef CNGGPS
#define CNGGPS

#include <string>

/**
 * \brief Creatively Named Grammar Guided Genetic Programming System
 * \author Stephen E. Baker
 * \date 2006-2007
 * \version 1.0
 *
 * The main class, which specifies the event loop coordinating the GP system.
 */
class CNGGGPS
{
	public:

	/**
	 * Runs the GP System.
	 *
	 * \param[in] strName
	 * 	The name of the experiment, used as a prefix for all the files
	 * 	created by the run.
	 * \param[in] iRuns
	 * 	The number of runs or trials to perform
	 * \param[in] iPopulationSize
	 * 	The number of programs to consider in each generation
	 * \param[in] iGenerations
	 * 	The number of selection, reproduction, evaluation cycles to
	 * 	perform in each run.
	 * \param[in] dMutationRate
	 * 	The proportion of the population to be mutated in each
	 * 	generation.
	 * \param[in] dCrossoverRate
	 * 	The proportion of the population to have crossover applied to
	 * 	them in each generation.
	 * \param[in] iMinDepth
	 * 	The minimum depth a symbol tree must obtain before being
	 * 	considered valid.
	 * \param[in] iMaxDepth
	 * 	The maximum depht a symbol tree may be to still be considered
	 * 	valid.
	 * \param[in] iTournamentSize
	 * 	The number of individuals randomly chosen from the population
	 * 	of which the best is passed on to the next generation.
	 */
	CNGGGPS(std::string strName, int iRuns = 1, int iPopulationSize = 100, int iGenerations = 50, double dMutationRate = 0.5, double dCrossoverRate = 0.5, int iMinDepth = 0, int iMaxDepth = 30, int iTournamentSize = 3, int iTries = 5);

	~CNGGGPS();
};

#endif

