#ifndef CNGGGPS_CONTEXT
#define CNGGGPS_CONTEXT

#include <map>
#include <string>
#include <vector>
#include "Result.hpp"

/**
 * \brief The program execution context or machine.
 * \author Stephen E. Baker
 * \date 2006-2007
 * \version 1.0
 *
 * This class allows you to pass values to arguments in the execution phase of
 * an individual.  It may easily be extended to allow programs to use
 * side-effects in their execution.
 */
class Context
{
public:
	/**
	 * Creates a new context
	 */
	Context();

	/**
	 * Removes all argument/value pairs from the context.  If the context
	 * has been extended to allow side-effects, this method should restore
	 * it to it's original state
	 */
	void clear();

	/**
	 * Give the context an argument value pair, to be passed into the
	 * program.  For example if the program contained the argument "x",
	 * and you wanted x to be interpreted as 10 you would pass in
	 * setArgumentValue("x", new Result(10));
	 *
	 * \param[in] strArgument
	 * 	The key or formal parameter
	 * \param[in] resValue
	 * 	The value or actual parameter
	 * \sa
	 * 	Result
	 */
	void setArgumentValue(std::string strArgument, Result* resValue);

	/**
	 * Returns the value or actual parameter matching the given key.
	 *
	 * \param[in] strArgument
	 * 	The key or formal parameter to match up against
	 */
	Result* getArgumentValue(std::string strArgument);

	/**
	 * Returns a vector of all the keys or formal parameters included
	 * in the context
	 */
	std::vector<std::string> getKeys() const;
	
	~Context();
private:
	/**
	 * A mapping which includes all key-value pairs
	 */
	std::map<std::string,Result*> mapArguments;
};

#endif

