#include <typeinfo>
#include <iostream>
#include <vector>
#include "Crossover.hpp"

using namespace std;

Crossover::Crossover(int iTries)
{
	this->iTries = iTries;
}

bool Crossover::crossover(Individual &ind1, Individual &ind2)
{
	Grammar *p1 = NULL;
	Grammar *p2 = NULL;
	vector<Grammar*>* pvpgramExpression1 = NULL;
	vector<Grammar*>* pvpgramExpression2 = NULL;
	int iRule1;
	int iRule2;
	int iFlag = false;
	
	//Pick symbols from the individuals which would cross to produce
	//new valid individuals
	for(int iTry = 0; !iFlag || (iTry < iTries); iTry++)
	{
		p1 = ind1.getRandomNode();
		p2 = ind2.getRandomNodeMatching(p1->getGrammarID());

		if(	!((p2 == NULL) || 
			((p1->getDepth() + getHeight(p2)) < ind1.getMinDepth()) || 
			((p1->getDepth() + getHeight(p2)) > ind1.getMaxDepth()) || 
			((p2->getDepth() + getHeight(p1)) < ind2.getMinDepth()) ||
			((p2->getDepth() + getHeight(p1)) > ind2.getMaxDepth())))
		{
			iFlag = true;
		}
	}

	if(iFlag == false) 
		return false;

	iRule1 = p1->getExpression(pvpgramExpression1);
	iRule2 = p2->getExpression(pvpgramExpression2);

	p1->setExpression(pvpgramExpression2, iRule2);
	p2->setExpression(pvpgramExpression1, iRule1);

	fixDepth(p1, p1->getDepth());
	fixDepth(p2, p2->getDepth());

	ind1.formNodeList();
	ind2.formNodeList();

	return true;
}

int Crossover::getHeight(Grammar* pgramNode)
{
	int iMax=0;
	int iHeight=0;

	for(int i = 0; i < pgramNode->getNumArguments(); i++)
	{
		iHeight = getHeight(pgramNode->getArgument(i));
		if(iHeight > iMax)
		{
			iMax = iHeight;
		}
	}
	return iMax+1;
}

void Crossover::fixDepth(Grammar* pgramNode, int iDepth)
{
	pgramNode->setDepth(iDepth);

	for(int i = 0; i < pgramNode->getNumArguments(); i++)
	{
		fixDepth(pgramNode->getArgument(i), iDepth+1);
	}
}

Crossover::~Crossover()
{}

