#ifndef CNGGGPS_CROSSOVER
#define CNGGGPS_CROSSOVER

#include "Grammar.hpp"
#include "Individual.hpp"

/**
 * \brief Performs crossover on individual pairs
 * \author Stephen E. Baker
 * \date 2006-2007
 * \version 1.0
 */
class Crossover
{
public:
	/**
	 * Creates a new object capable of performing crossover
	 *
	 * \param[in] iTries
	 * 	The number of attempted swaps which may come back as invalid 
	 * 	before the crossover is aborted
	 */
	Crossover(int iTries=5);

	/**
	 * Swaps a subtree in ind1 with a subtree in ind2 where the parent
	 * grammar symbol is the same (and thus the swap is valid).  If the
	 * swap is invalid (ie. violates depth restrictions on either
	 * individual, another random swap is made.
	 *
	 * \return
	 * 	- true - the crossover was performed successfully
	 * 	- false - the try limit was exceeded without a valid crossover
	 */
	bool crossover(Individual &ind1, Individual &ind2); 
	
	~Crossover();
private:
	/**
	 * Number of attemps which can be made before aborting
	 */
	int iTries;

	/**
	 * Calculates the height of a particular node.  This is done
	 * recursively.  If the depth of of a node + the height of the node
	 * it is being swaped with is not within the valid depth range the swap
	 * will be invalid
	 *
	 * \param[in] pgramNode
	 * 	The node for which the height is calculated
	 * \return
	 * 	The maximum distance from this node to any leaf node
	 */
	int getHeight(Grammar* pgramNode);

	/**
	 * Corrects the depth associated with the subtree of a given node.
	 *
	 * \param[in,out] pgramNode
	 * 	The parent of the subtree for which the depth is to be
	 * 	corrected
	 * \param[in]	  iDepth
	 * 	The actual depth of pgramNode
	 */
	void fixDepth(Grammar* pgramNode, int iDepth);
};

#endif

