#ifndef CNGPPS_FUNCTION
#define CNGPPS_FUNCTION

#include <string>
#include <vector>
#include "Result.hpp"
#include "Grammar.hpp"

/**
 * \brief Function abstract class
 * \author Stephen E. Baker
 * \date 2006-2007
 * \version 1.0
 *
 * Classes which extend this class are grammar terminals.  Unlike other
 * grammar symbols these appear in the phenotype (actual program) and
 * perform some form of calculation.
 */
class Function: public Grammar
{
	public:
	Function();

	/**
	 * Add an argument to the function, for example in addion
	 * each of the values to be added together would be specified
	 * with this function.
	 *
	 * \param[in] pres
	 * 	An argument for the function.
	 * \sa
	 * 	Result
	 */
	void addArgument(Result* pres);

	/**
	 * Removes all the arguments.  This should be called by the parent
	 * symbol after executing the function
	 */
	void clearArguments();

	/**
	 * Adds the functions strGrammarID to the string representation
	 * of the program.
	 *
	 * \sa
	 *	Grammar::strGrammarID
	 * \sa
	 * 	Grammar::toString
	 */
	virtual std::string toString(Context &context);
	
	virtual ~Function();
	
	protected:

	/**
	 * The arguments to be used in the funtion.
	 */
	std::vector<Result*> vpresArguments;
};

#endif

