#ifndef CNGGGPS_GRAMMAR_START
#define CNGGGPS_GRAMMAR_START

#include <sstream>
#include "Result.hpp"
#include "Grammar.hpp"

/**
 * \brief Start Symbol
 * \author Stephen E. Baker
 * \date 2006-2007
 * \version 1.0
 *
 * BNF requires a start symbol.  This symbol is used to begin every program.
 */
class GStart: public Grammar
{
public:
	/**
	 * Creates a new start symbol, presumably to begin a new program.
	 */
	GStart();

	/**
	 * Begins execution of the program by calling this method on it's
	 * child and passing back the value.
	 *
	 * \sa
	 * 	Grammar::execute
	 */
	Result* execute(Context &context);

	/**
	 * Adds the outer most text to the string representation of the
	 * expression.  Because CNGGGPS generates LISP programs, this
	 * adds a "(lambda(args)(" to the beginning and a ")" to the end.
	 *
	 * \param[in] context
	 *
	 * \sa
	 * 	Grammar::toString
	 */
	std::string toString(Context &context);

	/**
	 * \sa
	 * 	Grammar::clone
	 */
	Grammar* clone();

	~GStart();
};


#endif
