#ifndef CNGGGPS_GRAMMAR_RULES
#define CNGGGPS_GRAMMAR_RULES

#include <vector>
#include <string>
#include "Grammar.hpp"

/**
 * \brief Grammar Specification
 * \author Stephen E. Baker
 * \date 2006-2007
 * \version 1.0
 *
 * Class in which the grammar is defined.  The class is immuteable and there is
 * no need to instantiate objects of it.
 */
class GrammarRules
{
public:
	/**
	 * Returns a valid list of child symbols given a particular symbols.
	 *
	 * \param[in] strGrammarID
	 * 	A string specifying the symbol for which the child symbols are
	 * 	being queried.
	 * \param[in] uiType
	 * 	A flag specifying if there is any preference for a particular
	 * 	sort of rule to be chosen.
	 * \param[out] pvgramExpression
	 * 	The child symbol list that was chosen.
	 * \return
	 * 	The rule number chosen.
	 */
	static int getExpressionFor(std::string strGrammarID, unsigned int uiType, std::vector<Grammar*>* pvgrampExpression);

	/**
	 * flag to return symbols which lead to leaf nodes without recursive
	 * steps.
	 */
	static const unsigned int TERMINAL = 1;

	/**
	 * flag to return symbols which contain recursive steps.
	 */
	static const unsigned int NON_TERMINAL = 2;

	/**
	 * flag to return any symbols corresponding to the grammar 
	 * specification.
	 */
	static const unsigned int ANY = 3; 

private:
	/**
	 * Picks a rule randomly according to the desired features.
	 *
	 * \param[in] iTotalRules
	 * 	The number of rules corresponding to the symbol.
	 * \param[in] iTerminalRules
	 * 	The number of rules which are guarenteed to lead
	 * 	to terminal states in a finite number of steps.
	 * \param[in] uiType
	 * 	Flag determining the prefered type of rule to choose.
	 * \return
	 * 	A random rule number within the desired range if any exist.
	 */
 	static int pickRule(int iTotalRules, int iTerminalRules, unsigned int uiType);
};

#endif

