#include <iomanip>
#include <fstream>
#include "Log.hpp"

#define LOGAT(r,g) ((vvle.at(g)).at(r))

using namespace std;

Log::Log(string strName, int iRuns, int iGenerations)
{
	this->iRuns = iRuns;
	this->iGenerations = iGenerations;

	vvle.resize(iGenerations);
	for(vector<vector<SLogEntry> >::iterator it = vvle.begin(); it != vvle.end(); it++)
	{
		it->resize(iRuns);
	}

	this->strName = strName;
}

void Log::logParameters(int iPopulationSize, double dCrossoverRate, double dMutationRate, int iTries, int iMinDepth, int iMaxDepth, int iTournamentSize)
{
	ofstream ofsLog(string(strName+".par").c_str());

	ofsLog << "Runs: " << iRuns << endl;
	ofsLog << "Generations: " << iGenerations << endl;
	ofsLog << "Population Size: " << iPopulationSize << endl;
	ofsLog << "Crossover Rate: " << dCrossoverRate << endl;
	ofsLog << "Mutation Rate: " << dMutationRate << endl;
	ofsLog << "Mutation/Crossover attempts: " << iTries << endl;
	ofsLog << "Minimum Depth: " << iMinDepth << endl;
	ofsLog << "Maximum Depth: " << iMaxDepth << endl;
	ofsLog << "Tournament Size: " << iTournamentSize << endl;
}

void Log::addEntry(int iRun, int iGeneration, double dBestFitness, double dAvgFitness, int iBestHits, double dAvgHits)
{
	LOGAT(iRun,iGeneration).dBestFitness = dBestFitness;
	LOGAT(iRun,iGeneration).dAvgFitness = dAvgFitness;
	LOGAT(iRun,iGeneration).iBestHits = iBestHits;
	LOGAT(iRun,iGeneration).dAvgHits = dAvgHits;
}

void Log::logBest(vector<Individual*> vpindBest)
{
	ofstream ofsLog(string(strName+".bst").c_str());

	for(int iRun = 0; iRun < iRuns; iRun++)
	{
		ofsLog << "Run " << iRun+1 << " of " << iRuns << endl;
		ofsLog << "Fitness: " << vpindBest.at(iRun)->getFitness() << endl;
		ofsLog << "Hits: " << vpindBest.at(iRun)->getHits() << endl;
		ofsLog << vpindBest.at(iRun)->toString() << endl;
		ofsLog << endl;
	}
}

Log::~Log()
{
	dumpToFile();
}

void Log::dumpToFile()
{
	ofstream ofsLog(string(strName+".csv").c_str());
	ofsLog.setf(ios::showpoint);

	ofsLog << "Best\tAverage" << endl;

	for(vector<vector<SLogEntry> >::iterator itGen = vvle.begin(); itGen != vvle.end(); itGen++)
	{
		for(vector<SLogEntry>::iterator itEntry = itGen->begin(); itEntry != itGen->end(); itEntry++)
		{
			ofsLog << itEntry->dBestFitness << "\t" << itEntry->dAvgFitness << "\t";
		}
		ofsLog << endl;
	}
}
